// RemoteSensorClient.java

import ch.aplu.util.*;
import java.io.*;
import javax.bluetooth.RemoteDevice;
import ch.aplu.bluetooth.*;
import java.awt.Color;
import javax.swing.JOptionPane;

public class RemoteSensorClient extends GPanel implements ExitListener
{
  private static final double ymax = 10;
  private static final int tmax = 10000; // ms
  private static String serverName = "";
  private final String serviceName = "RemoteSensor";
  private boolean isConnected = false;
  private final int disconnectTag = Integer.MAX_VALUE;
  private BluetoothClient bc;
  private DataInputStream dis;
  private DataOutputStream dos;

  public RemoteSensorClient()
  {
    super(0, tmax, -ymax, ymax);
    title("RemoteSensorClient");
    enableRepaint(false);
    addStatusBar(40);
    addExitListener(this);
    initPanel();
    setStatusText("Select server's Bluetooth name");
    String prompt = "Enter Server's Bluetooth Name";
    do
    {
      serverName = JOptionPane.showInputDialog(null, prompt, serverName);
      if (serverName == null)
        System.exit(0);
    }
    while (serverName.trim().length() == 0);
    serverName = serverName.trim();

    setStatusText("Trying to find paired device '" + serverName + "'");

    // Try to get device from paired devices database
    RemoteDevice rd = BluetoothFinder.searchPreknownDevice(serverName);
    if (rd == null)
    {
      setStatusText("Device '" + serverName + "' not paired. Perform pairing.");
      return;
    }
    setStatusText("Paired device found. Searching service now...");
    bc = new BluetoothClient(rd, serviceName);
    bc.setVerbose(true);
    setStatusText("Connecting  to '" + serverName + "' ...");
    if (!bc.connect())
    {
      setStatusText("Connection to '" + serverName + "' failed.");
      return;
    }
  
    String msg = "Connection to '" + serverName + "' established.";
    setStatusText(msg);
    dis = new DataInputStream(bc.getInputStream());
    dos = new DataOutputStream(bc.getOutputStream());
    isConnected = true;
    int txOld = 0;
    String timeStrOld = "";
    try
    {
      while (true)
      {
        int t = dis.readInt();
        int value = dis.readInt();

        if (t == disconnectTag || value == disconnectTag)
          throw(new IOException("Got disconnect tag"));  

        String timeStr = String.format("%5.1f", t / 1000.0);
        if (!timeStr.equals(timeStrOld))
        {  
          setStatusText(msg + "  Time elapsed: " + timeStr);
          timeStrOld = timeStr;
        }
        int tx = t % tmax;  // Reduced time 
        if (tx < txOld)
          initPanel();
        draw(tx, value / 1000.0);
        repaint();
        txOld = tx;
      }
    }
    catch (IOException ex)
    {
      setStatusText("Connection to '" + serverName + "' lost.");
    }
  }

  private void initPanel()
  {
    clear();
    color(Color.lightGray);
    for (double y = -ymax; y <= ymax; y += 2)
      line(0, y, tmax, y);
    for (double t = 0; t <= tmax; t += 2000)
      line(t, -ymax, t, ymax);

    move(0, 0);
    color(Color.black);
    repaint();
  }


  public void notifyExit()
  {
    if (isConnected)
    {
      try
      {
        dos.writeInt(disconnectTag);
        dos.flush();
      }
      catch (IOException ex)
      {
      }
      bc.disconnect();
      delay(1000);
    }
    System.exit(0);
  }

  public static void main(String[] args)
  {
    if (args.length > 0)
      serverName = args[0];
    new RemoteSensorClient();
  }

}
