// ProgressBar.java

package ch.aplu.tut;

import ch.aplu.android.*;
import android.graphics.*;

public class ProgressBar
{
  private GameGrid gg;
  private Actor barActor = null;
  private TextActor textActor = null;
  private Location loc;
  private int width;
  private int height;

  public ProgressBar(GameGrid gg, Location loc, int width, int height)
  {
    this.gg = gg;
    this.loc = loc;
    this.width = width;
    this.height = height;
  }

  public void setValue(int value)
  {
    value = Math.max(0, Math.min(100, value));  // 0..100
    if (barActor != null)
    {
      barActor.removeSelf();
      textActor.removeSelf();
    }
    barActor = new Actor(createActor(width, height, value));
    gg.addActorNoRefresh(barActor, loc);
    textActor = new TextActor(value + "%");
    int w = textActor.getTextWidth() / 2;
    textActor.setLocationOffset(new android.graphics.Point(-w, 0));
    gg.addActor(textActor, loc);
  }

  private Bitmap createActor(int width, int height, int value)
  {
    int length = value * (width - 1) / 100;
    GGBitmap bm = new GGBitmap(width + 1, height + 1);
    bm.setPaintColor(Color.WHITE);
    bm.drawRectangle(new Point(0, 0), new Point(width, height));
    bm.setPaintColor(Color.BLUE);
    bm.fillRectangle(new Point(1, 1), new Point(length, height - 1));
    return bm.getBitmap();
  }

}
