// BtClientApp.java
// Data exchange with BtServerApp or BtServerSE

package ch.aplu.btclient;

import android.bluetooth.*;
import ch.aplu.android.*;
import ch.aplu.android.bluetooth.*;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

public class BtClientApp extends GameGrid
  implements GGNavigationListener
{
  // ------------- Inner class TransceiverThread ---------------
  private class TransceiverThread extends Thread
  {
    public void run()
    {
      delay(2000);  // Wait until server is ready
      try
      {
        for (int n = 1; n <= 100; n++)
        {
          dos.writeInt(n);
          dos.flush();
          c.println("Sent: " + n);

          // Blocking, throws IOException
          // when server closes connection or BluetoothClient.disconnect() is called
          int k = dis.readInt();
          c.println("Got: " + k);
          if (k == disconnectTag)
            throw new IOException("Got disconnect notification");
        }
        dos.writeInt(disconnectTag);
        dos.flush();
      }
      catch (IOException ex)
      {
        c.println("Transceiver thread: " + ex.getMessage());
      }
      delay(2000);
      bc.disconnect();
      c.println("Transceiver thread terminated");
      c.println("Streams and socket closed");
    }

  }
  // ------------- End of inner class -----------------------

  private final int disconnectTag = 9999;
  private BluetoothClient bc;
  private DataInputStream dis;
  private DataOutputStream dos;
  private GGConsole c;

  public void main()
  {
    c = GGConsole.init();
    addNavigationListener(this);
    connect();
  }

  private void connect()
  {
    c.println("Asking for server Bluetooth name");
    String serverName = askName();
    c.println("Searching for server '" + serverName + "' ...");
    BluetoothDevice serverDevice = searchDevice(serverName);
    if (serverDevice != null)
    {
      c.println("Trying to connect to server...");
      bc = new BluetoothClient(serverDevice);
      boolean rc = bc.connect();
      if (!rc)
      {
        c.println("Connection failed.");
        return;
      }
      c.println("Connection established.");

      dis = new DataInputStream(bc.getInputStream());
      dos = new DataOutputStream(bc.getOutputStream());
      c.println("Starting transceiver thread");
      new TransceiverThread().start();
    }
  }

  // Is called by the system
  public void onPause()
  {
    if (bc != null && bc.isConnected())
    {
      try
      {
        dos.writeInt(disconnectTag);
        dos.flush();
      }
      catch (IOException ex)
      {
      }
      bc.disconnect();
    }
    super.onPause();
  }

  private String askName()
  {
    GGPreferences prefs = new GGPreferences(this);
    String oldName = prefs.retrieveString("BluetoothName");
    String newName = null;
    while (newName == null)
    {
      newName = GGInputDialog.show(c.getActivity(), "Search Device",
        "Enter Bluetooth Name",
        oldName == null ? "" : oldName);
    }
    prefs.storeString("BluetoothName", newName);
    return newName;
  }

  // Searches for device in set of paired devices. Returns null, if search fails.
  private BluetoothDevice searchDevice(String btName)
  {
    BluetoothAdapter adapter =
      BluetoothDiscovery.getBluetoothAdapter(this);
    if (adapter == null)
    {
      c.println("Bluetooth not supported");  // e.g. in emulator
      return null;
    }
    if (!adapter.isEnabled())
    {
      requestBluetoothEnable();
      if (!adapter.isEnabled())
      {
        c.println("Can' enable Bluetooth");
        return null;
      }
    }
    c.println("Searching for paired device '" + btName + "'...");
    BluetoothDiscovery bd = new BluetoothDiscovery(this);
    BluetoothDevice device = bd.getPairedDevice(btName);
    if (device != null)
      c.println("Paired device '" + btName + "' found.");
    else
      c.println("Please pair with device '" + btName + "'");
    return device;
  }

  public void navigationEvent(GGNavigationEvent event)
  {
    showToast("Not implemented");
  }

}
