// RemoteDiscovery.java
// Searching for all Bluetooth devices

package ch.aplu.btex;

import ch.aplu.android.*;
import ch.aplu.android.bluetooth.*;
import android.bluetooth.*;
import java.util.Set;

public class RemoteDiscovery extends GameGrid
  implements BluetoothDiscoveryListener
{
  private GGConsole c;
  private volatile boolean isDiscovering;
   
  public void main()
  {
    c = GGConsole.init();
    BluetoothAdapter bluetoothAdapter = 
      BluetoothDiscovery.getBluetoothAdapter(this);
    if (!getBluetoothAdapterInfo(bluetoothAdapter))
      return;
    isDiscovering = true;
    c.println("Starting Bluetooth discovery...");
    BluetoothDiscovery bf = new BluetoothDiscovery(this);
    bf.startBluetoothDiscovery(this);
    while (isDiscovering)
      delay(10);
    c.println("All done");
  }

  public void bluetoothDeviceDiscovered(BluetoothDevice device)
  {
    c.println("Found device name: " + device.getName());
    c.println("Address: " + device.getAddress());
    // Service inquiry with getUuids() only available since API level 15
  }

  public void bluetoothDiscoveryFinished()
  {
    c.println("Discovery finished");
    isDiscovering = false;
  }

  private boolean getBluetoothAdapterInfo(BluetoothAdapter adapter)
  {
    c.println("Checking if Bluetooth is supported...");
    if (adapter == null)
    {
      c.println("Failed.");
      return false;
    }
    c.println("OK. Checking if Bluetooth is enabled...");
    if (!adapter.isEnabled())
    {
      c.println("No. Please enable Bluetooth device");
      return false;
    }
    c.println("OK. Bluetooth friendly name: " + adapter.getName());
    c.println("Bluetooth MAC address: " + adapter.getAddress());
    c.println("Paired Bluetooth devices:");
    Set<BluetoothDevice> pairedDevices = adapter.getBondedDevices();
    if (pairedDevices.size() == 0)
      c.println("None");
    else
    {
      for (BluetoothDevice device : pairedDevices)
        c.println(device.getName() + " (" + device.getAddress() + ")");
    }
    return true;
  }
}
