// CardEx03.java
// Dragging cards

package ch.aplu.cardex03;

import android.graphics.*;
import ch.aplu.jcardgame.*;
import ch.aplu.android.*;

public class CardEx03 extends CardGame implements GGActorTouchListener
{
  public enum Suit
  {
    SPADES, HEARTS, DIAMONDS, CLUBS
  }

  public enum Rank
  {
    ACE, KING, QUEEN, JACK, TEN
  }

  private Deck deck;
  private Hand talon;
  private Hand hand;
  private final Location talonLocation = new Location(300, 300);
  private final Location handLocation = new Location(300, 100);

  public CardEx03()
  {
    super(Color.rgb(20, 80, 0), Color.WHITE, BoardType.HORZ_SQUARE, windowZoom(600)); 
  }

  public void main()
  {
    deck = new Deck(Suit.values(), Rank.values(), "cover");
    Hand[] hands = deck.dealingOut(1, 2);  // only two cards in hand
    hand = hands[0];
    hand.setView(this, new RowLayout(handLocation, 400));
    talon = hands[1];
    talon.setView(this, new StackLayout(talonLocation));
    hand.draw();
    talon.draw();
    takeFromTalon();
    showToast("Drag cards from card talon to the hand");
  }

  private void insertInHand(Actor actor)
  {
    actor.removeSelf();
    Card card = ((CardActor)actor).getCard();
    hand.insert(card, false);
    hand.sort(Hand.SortType.RANKPRIORITY, true);
    takeFromTalon();
  }

  private void takeFromTalon()
  {
    Card top = talon.getLast();
    if (top == null)  // talon empty
      return;
    talon.remove(top, false);  // Remove from talon and game grid
    addActor(top.getCardActor()); // Reinsert into game grid
  }

  private void addActor(CardActor cardActor)
  {
    addActor(cardActor, new Location(talonLocation));
    cardActor.addActorTouchListener(this,  GGTouch.drag | GGTouch.release, true);
  }

  public void actorTouched(Actor actor, GGTouch touch, Point spot)
  {
    switch (touch.getEvent())
    {
       case GGTouch.drag:
        actor.setLocation(new Location(
          pixelToVirtual(touch.getX()),
          pixelToVirtual(touch.getY())));
        if (actor.getLocation().getDistanceTo(handLocation) < 50)
          insertInHand(actor);
        break;

      case GGTouch.release:
        insertInHand(actor);
        break;
    }
  }

}