// Puck.java

package ch.aplu.movingpuck;

import android.graphics.*;
import ch.aplu.android.*;

class Puck extends Actor
{
  private MovingPuck app;
  // Physical variables (all in physical units)
  private double x, y;  // Position (m)
  private double vx, vy;  // Velocity (m/s)
  private double ax, ay;  // Acceleration (m/s^2)
  private final double dt = 0.05;  // Integration interval (s)
  private final double f = 0.2; // Friction (s^-1)

  public Puck(MovingPuck app)
  {
    super("puck");
    this.app = app;

    // Physical initial conditions:
    x = 0;
    y = 0;
    vx = 0;
    vy = 0;
  }

  public void act()
  {
    double[] a = app.sensor.getAcceleration(0);
    double gx = -a[4];
    double gy = a[3];

    // New acceleration:
    ax = gx - f * vx;
    ay = gy - f * vy;

    // New velocity:
    double vxNew = vx + ax * dt;
    double vyNew = vy + ay * dt;

    // New position
    double xNew = x + vxNew * dt;
    double yNew = y + vyNew * dt;

    // Test if in area
    if (!isInside(xNew, yNew))  // No->set speed to zero
    {
      vx = 0;
      vy = 0;
      return;  // and quit
    }

    setLocation(new Location(app.p.toPixelX(xNew), app.p.toPixelY(yNew)));
    app.p.line(new PointD(x, y), new PointD(xNew, yNew));

    vx = vxNew;
    vy = vyNew;
    x = xNew;
    y = yNew;
  }

  private boolean isInside(double x, double y)
  {
    double R = 0.9 * app.courtSize / 2 - app.ballRadius;
    return x * x + y * y <= R * R;
  }

}
