// NxtActor.java

package ch.aplu.nxtcommander;

import ch.aplu.android.*;

class NxtActor extends Actor
{
  private enum State
  {
    STOP, FORWARD, BACKWARD, LEFT, RIGHT
  };
  private final double linearSpeed = 1;
  private final double angularSpeed = 0.70;
  private State state = State.STOP;
  private double xold, yold;

  public NxtActor()
  {
    super(true, "roby");
  }

  public void stop()
  {
    state = State.STOP;
  }

  public void forward()
  {
    state = State.FORWARD;
  }

  public void backward()
  {
    state = State.BACKWARD;
  }

  public void left()
  {
    state = State.LEFT;
  }

  public void right()
  {
    state = State.RIGHT;
  }

  public void init()
  {
    xold = getX();
    yold = getY();
    state = State.STOP;
  }

  public void act()
  {
    double xnew, ynew;

    switch (state)
    {
      case FORWARD:
        xnew = xold + linearSpeed * Math.cos(Math.toRadians(getDirection()));
        ynew = yold + linearSpeed * Math.sin(Math.toRadians(getDirection()));
        xold = xnew;
        yold = ynew;
        setLocation(new Location((int)xnew, (int)ynew));
        break;
      case BACKWARD:
        xnew = xold - linearSpeed * Math.cos(Math.toRadians(getDirection()));
        ynew = yold - linearSpeed * Math.sin(Math.toRadians(getDirection()));
        xold = xnew;
        yold = ynew;
        setLocation(new Location((int)xnew, (int)ynew));
        break;
      case LEFT:
        setDirection(getDirection() - angularSpeed);
        break;
      case RIGHT:
        setDirection(getDirection() + angularSpeed);
        break;
    }
  }

  public Location getSensorLocation()
  {
    int d = 10;
    double xSendor = getX() + d * Math.cos(Math.toRadians(getDirection()));
    double ySendor = getY() + d * Math.sin(Math.toRadians(getDirection()));
    return new Location((int)xSendor, (int)ySendor);
  }
}
