// NxtSensing.java
// Remote sensing (light sensor)

package ch.aplu.nxtsensing;

import ch.aplu.android.*;
import ch.aplu.android.nxt.*;
import ch.aplu.util.*;
import java.text.DecimalFormat;

public class NxtSensing extends GameGrid implements NxtConnectionListener
{
  private final int period = 200; // ms 
  private NxtRobot robot;
  private volatile boolean isRunning = true;
  
  public NxtSensing()
  {
    super(21, 21, 0);
    setScreenOrientation(PORTRAIT);
  }

  public void main()
  {
    getBg().clear(android.graphics.Color.BLUE);
    GGTextField tf1 = 
      new GGTextField("Remote Sensing Luminosity", new Location(0, 1), true);
    tf1.show();
    GGTextField tf2 = new GGTextField(new Location(0, 4), true);
    tf2.show();
    robot = new NxtRobot(this);
    robot.addNxtConnectionListener(this);
    if (robot.connect() != NxtRobot.ConnectState.CONNECTED)
    {
      tf2.setText("Connection failed");
      return;
    }
    LightSensor ls = new LightSensor();
    robot.addPart(ls);
    HiResAlarmTimer timer = new HiResAlarmTimer(1000 * period);
    double time = 0;
    DecimalFormat df = new DecimalFormat("0.0");
    while (isRunning)
    {
      timer.start();
      int value = ls.getValue();
      tf2.setText("At " + df.format(time) + " s: lum = " + value);
      while (timer.isRunning())
        delay(1);
      time += period / 1000.0;
    }
    tf2.setText("Connection to NXT lost");
  }
  
  public void onPause()
  {
    if (robot != null)
      robot.exit();
    super.onPause();
  }
  
  public void notifyConnection(boolean connected)
  {
    if (!connected)
      isRunning = false;
  }
}


