// SensorGraph.java

package ch.aplu.sensor.graph;

import android.graphics.Color;
import android.graphics.Typeface;
import android.hardware.*;
import ch.aplu.android.*;
import ch.aplu.util.*;

public class SensorGraph extends GameGrid
  implements GGSensorListener
{
  private GGStatusBar status;
  private GGPanel p;
  private float[] acc = new float[3];
  private GGSensor accelerometer;
  private final int period = 100;  // ms
  private double tmax = 10;
  private double amax = 10;

  public SensorGraph()
  {
    super(WHITE, false, true, null);
    setScreenOrientation(LANDSCAPE);
    status = addStatusBar(30);
  }

  public void main()
  {
    setWakeLockEnabled(true);
    status.setTypeface(Typeface.MONOSPACE);
    p = getPanel(0, tmax, -amax, amax);
    initPanel();
    p.setAutoRefreshEnabled(false);
    accelerometer = new GGSensor(this, Sensor.TYPE_ACCELEROMETER,
      SensorManager.SENSOR_DELAY_UI);
    accelerometer.addSensorListener(this);
    accelerometer.setHighPassFilter(10, 1);
    HiResAlarmTimer timer = new HiResAlarmTimer(1000 * period);
    timer.start();
    double t = 0;
    double txOld = 0;

    while (true)
    {
      double a;
      if (isOrientationTablet())
        a = acc[0];
      else
        a = acc[1];
      status.setText(String.format("t = %3.1f s, ay = % 3.2f m/s^2", t, a));
      double tx = t % tmax;  // Reduced time
      if (tx < txOld)
        initPanel();
      txOld = tx;
      p.draw(tx, a);
      refresh();
      t += period / 1000.0;
      while (timer.isRunning())
        delay(1);
      timer.start();
    }
  }

  private void initPanel()
  {
    p.clear();
    p.color(Color.rgb(50, 50, 10));
    for (double a = -amax + 2; a < amax; a += 2)
      p.line(0, a, tmax, a);
    for (double t = 2; t < tmax; t += 2)
      p.line(t, -amax, t, amax);

    p.move(0, 0);
    p.color(WHITE);
  }

  public void sensorChanged(float[] values, int sensorType)
  {
    for (int i = 0; i < 3; i++)
      acc[i] = values[i];
  }

}
