// RemoteSensor.java

package ch.aplu.sensor.remote;

import ch.aplu.android.*;
import ch.aplu.android.bluetooth.*;
import android.bluetooth.*;
import android.graphics.Typeface;
import android.hardware.Sensor;
import android.hardware.SensorManager;
import java.io.*;
import ch.aplu.util.*;

public class RemoteSensor extends GameGrid
  implements BluetoothServerListener, GGNavigationListener,
  GGSensorListener
{
  private class ReceiverThread extends Thread
  {
    private boolean isRunning;

    public void run()
    {
      isRunning = true;
      while (isRunning)
      {
        try
        {
          int data = dis.readInt();
          c.println("Got data: " + data);
          if (data == disconnectTag)
          {
            c.println("(disconnect notification)");
            isRunning = false;
            continue;
          }
        }
        catch (IOException ex)
        {
          c.println("IOExeption in blocking readInt()");
          isRunning = false;
        }
      }
      c.println("Receiver thread terminated");
      Monitor.wakeUp(); // Wake-up sleeping notifyConnection()
    }

  }

  private class AcquisitionThread extends Thread
  {
    private boolean isRunning;

    public void run()
    {
      isRunning = true;
      int value;
      int time = 0;
      long startTime;
      long timeInc;
      long offset = 1000000;  // Increases precision of period
    
      while (isRunning)
      {
        try
        {
          startTime = System.nanoTime();
          if (isOrientationTablet())
            value = (int)(1000 * acc[0]);
          else
            value = (int)(1000 * acc[1]);
          dos.writeInt(time);
          dos.writeInt(value);
          dos.flush();
//          L.i("time = " + time + " value = " + value);
          do
          {  
            timeInc = System.nanoTime() - startTime;
          }  
          while (timeInc < (1000000 * period - offset));
          time += timeInc / 1000000;
        }
        catch (IOException ex)
        {
          c.println("IOExeption while sending data");
          isRunning = false;
        }
      }
      c.println("Acquisition thread terminated");
    }

  }

  //
  private final String serviceName = "RemoteSensor";
  private final int disconnectTag = Integer.MAX_VALUE;
  private BluetoothAdapter adapter;
  private BluetoothServer bs;
  private DataInputStream dis;
  private DataOutputStream dos;
  private int nbConnections = 0;
  private boolean isClientConnected = false;
  private GGConsole c;
  private float[] acc = new float[3];
  private GGSensor accelerometer;
  private final int period = 50;  // ms

  public void main()
  {
    setWakeLockEnabled(true);
    accelerometer = new GGSensor(this, Sensor.TYPE_ACCELEROMETER,
      SensorManager.SENSOR_DELAY_FASTEST);
    accelerometer.addSensorListener(this);

    c = GGConsole.init();
    addNavigationListener(this);

    adapter = BluetoothDiscovery.getBluetoothAdapter(this);
    c.println("Creating server with Bluetooth name '"
      + adapter.getName() + "'");
    bs = new BluetoothServer(adapter, serviceName, this);
    c.println("Waiting for a Bluetooth client to connect ");
    c.println("exposing service '" + serviceName + "'");
  }

  public boolean notifyClientConnection(BluetoothDevice device,
    InputStream is, OutputStream os)
  {
    c.println("notifyClientConnection() starting");
    nbConnections++;
    c.println("Client '" + device.getName() + "' connected as # " + nbConnections);
    isClientConnected = true;
    c.println("Retrieving input and output streams");
    dis = new DataInputStream(is);
    dos = new DataOutputStream(os);
    c.println("Starting receiver thread");
    new ReceiverThread().start();
    c.println("Starting aquisition thread");
    new AcquisitionThread().start();
    c.println("notifyClientConnection() blocking now");
    Monitor.putSleep();  // Thread goes to sleep
    c.println("notifyClientConnection() wakes up");
    c.println("Lost client connection. Server will close streams and socket");
    c.println("Waiting for next client...");
    isClientConnected = false;
    return true;
  }

  // Is called by the system
  public void onPause()
  {
    if (isClientConnected)
      sendDisconnectTag();
    if (bs != null)
      bs.abort();
    super.onPause();
  }

  private void sendDisconnectTag()
  {
    try
    {
      dos.writeInt(disconnectTag);
      dos.flush();
    }
    catch (IOException ex)
    {
    }
  }

  public void navigationEvent(GGNavigationEvent event)
  {
    showToast("Not implemented");
  }

  public void sensorChanged(float[] values, int sensorType)
  {
    for (int i = 0; i < 3; i++)
      acc[i] = values[i];
  }

}
