// Sokoban.java

package ch.aplu.sokoban;

import ch.aplu.android.*;
import android.graphics.*;

public class Sokoban extends GameGrid implements GGSoftButtonListener
{
  // ------------- Inner classes -------------
  private class SokobanActor extends Actor
  {
    public SokobanActor()
    {
      super(true, "sokoban");  // Rotatable
    }
  }

  private class SokobanTarget extends Actor
  {
    public SokobanTarget()
    {
      super("target");
    }
  }

  private class SokobanStone extends Actor
  {
    public SokobanStone()
    {
      super("sokobanstone", 2); // Two sprites
    }
  }
  // ------------- End of inner classes ------
  
  private final static SokobanGrid grid = new SokobanGrid(0);
  private final static int nbHorzCells = grid.getNbHorzCells();
  private final static int nbVertCells = grid.getNbVertCells();
  private final int borderColor = Color.rgb(130, 10, 50);
  private SokobanStone[] stones = new SokobanStone[grid.getNbStones()];
  private SokobanTarget[] targets = new SokobanTarget[grid.getNbStones()];
  private SokobanActor sok;
  private boolean isFinished = false;

  
  public Sokoban()
  {
    super(nbHorzCells, nbVertCells, cellZoom(20), RED);
    addSoftButton(0, "Up");  // Must be done in ctor
    addSoftButton(1, "Dn");
    addSoftButton(2, "Lt");
    addSoftButton(3, "Rt");
    L.i("-------------------->ok");
  }

  public void main()
  {
    GGBackground bg = getBg();
    drawBoard(bg);
    drawActors();
    addSoftButtonListener(this);
  }

  private void drawActors()
  {
    int stoneIndex = 0;
    int targetIndex = 0;

    for (int y = 0; y < nbVertCells; y++)
    {
      for (int x = 0; x < nbHorzCells; x++)
      {
        Location location = new Location(x, y);
        int a = grid.getCell(location);
        if (a == 5) // Sokoban actor
        {
          sok = new SokobanActor();
          addActor(sok, location);
        }
        if (a == 3) // Stones
        {
          stones[stoneIndex] = new SokobanStone();
          addActor(stones[stoneIndex], location);
          stoneIndex++;
        }
        if (a == 4) // Targets
        {
          targets[targetIndex] = new SokobanTarget();
          addActor(targets[targetIndex], location);
          targetIndex++;
        }
      }
    }
    setPaintOrder(SokobanTarget.class);
  }

  private void drawBoard(GGBackground bg)
  {
    bg.clear(Color.rgb(230, 230, 230));
    bg.setPaintColor(Color.DKGRAY);
    for (int y = 0; y < nbVertCells; y++)
    {
      for (int x = 0; x < nbHorzCells; x++)
      {
        Location location = new Location(x, y);
        int a = grid.getCell(location);
        if (a == 1 || a == 3 || a == 4 || a == 5)
        {
          bg.fillCell(location, Color.LTGRAY);
          Point center = toPoint(location);
          bg.drawLine(new Point(center.x - 15, center.y - 15),
            new Point(center.x + 15, center.y + 15));
          bg.drawLine(new Point(center.x - 15, center.y + 15),
            new Point(center.x + 15, center.y - 15));
        }
        if (a == 2)  // Border
          bg.fillCell(location, borderColor);
      }
    }
  }
  
  public void buttonPressed(int buttonNb)
  {
  } 
  
  public void buttonReleased(int buttonNb)
  {
  }
  
  public void buttonClicked(int buttonNb)
  {
  }
 
  public void buttonRepeated(int buttonNb)
  {
    if (isFinished)
      return;
    Location next = null;
    switch (buttonNb)
    {
      case 2:
        next = sok.getLocation().getNeighbourLocation(Location.WEST);
        sok.setDirection(Location.WEST);
        break;
      case 0:
        next = sok.getLocation().getNeighbourLocation(Location.NORTH);
        sok.setDirection(Location.NORTH);
        break;
      case 3:
        next = sok.getLocation().getNeighbourLocation(Location.EAST);
        sok.setDirection(Location.EAST);
        break;
      case 1:
        next = sok.getLocation().getNeighbourLocation(Location.SOUTH);
        sok.setDirection(Location.SOUTH);
        break;
    }
    if (next != null && canMove(next))
    {
      sok.setLocation(next);
    }
    refresh();
  }

  private boolean canMove(Location location)
  {
    // Test if try to move into border
    int c = getBg().getColor(location);
    if (c == borderColor)
      return false;
    else // Test if there is a stone
    {
      SokobanStone stone = (SokobanStone)getOneActorAt(location, SokobanStone.class);
      if (stone != null)
      {
        // Try to move the stone
        stone.setDirection(sok.getDirection());
        if (moveStone(stone))
          return true;
        else
          return false;
      }
    }
    return true;
  }

  private boolean moveStone(SokobanStone stone)
  {
    Location next = stone.getNextMoveLocation();
    // Test if try to move into border
    int c = getBg().getColor(next);
    if (c == borderColor)
      return false;

    // Test if there is another stone
    SokobanStone neighbourStone =
      (SokobanStone)getOneActorAt(next, SokobanStone.class);
    if (neighbourStone != null)
      return false;

    // Move the stone
    stone.setLocation(next);

    // Check if we are at a target
    if (getOneActorAt(next, SokobanTarget.class) != null)
      stone.show(1);
    else
      stone.show(0);
    return true;
  }

  
}