// SpiritLevel.java

package ch.aplu.spiritlevel;

import ch.aplu.android.*;
import android.graphics.*;
import android.hardware.SensorManager;

public class SpiritLevel extends GameGrid
  implements GGNavigationListener
{
  private int offset;
  private Actor bubble;
  private GGComboSensor sensor;
  private int rLimit;
  private boolean isInitDone = false;
  private boolean isBackPressed;

  public SpiritLevel()
  {
    super("gear", windowZoom(600));
  }

  public void main()
  {
    if (isInitDone)
    {
      showToast("SpiritLevel up again");
      doRun();
      return;
    }
    isInitDone = true;

    offset = getNbVertCells() / 2;
    rLimit = getNbVertCells() / 4;
    sensor = GGComboSensor.init(this, SensorManager.SENSOR_DELAY_FASTEST);
    bubble = new Actor("bubble");
    addActor(bubble, new Location(offset, offset));
    setSimulationPeriod(30);

    // Remove the standard behavior 
    // when the [BACK], [MENU] or [VOLUME] button is hit
    addNavigationListener(this);

    // Prevent actors to be removed from GameGrid
    setCleanupEnabled(false);

    doRun();
    showToast("SpiritLevel ready");
  }

  public void act()
  {
    // origin of x, y in center of window
    float[] a = sensor.getOrientation(3);
    Point p = new Point(
      (int)(-0.02 * a[5] * getNbVertCells()), // Roll
      (int)(0.02 * a[4] * getNbVertCells())); // Pitch
    p = reduce(p);
    bubble.setLocation(new Location(offset + p.x, offset + p.y));
  }

  private Point reduce(Point p)
  {
    // Reduce point to limiting circle
    if (p.x * p.x + p.y * p.y > rLimit * rLimit)
      return new Point(
        (int)(rLimit * p.x / Math.sqrt(p.x * p.x + p.y * p.y)),
        (int)(rLimit * p.y / Math.sqrt(p.x * p.x + p.y * p.y)));
    else
      return p;
  }

  // Prevents killing of process
  public void onPause()
  {
    superOnPause();
    doPause();  // Must pause simulation cycling
  }

  // Callback when a device button is hit
  public void navigationEvent(GGNavigationEvent event)
  {
    switch (event)
    {
      case BACK_DOWN:
        if (!isBackPressed)
        {
          isBackPressed = true;
          showToast("Click back once more to exit");
          new Thread()
          {
            public void run()
            {
              delay(2000);
              isBackPressed = false;
            }

          }.start();
        }
        else
          finish();
        break;

      case MENU_DOWN:
        showToast("SpiritLevel by www.aplu.ch");
        break;
    }
  }

}
