// TcpFourInARow.java

package ch.aplu.tcpfourinarow;

import ch.aplu.android.*;
import ch.aplu.tcp.*;
import java.util.*;
import android.graphics.*;

public class TcpFourInARow extends GameGrid
  implements GGTouchListener, TcpNodeListener
{
  interface Command
  {
    char start = 's';
    char ready = 'r';
    char drag = 'd';
    char move = 'm';
    char over = 'o';
  }
  private static final int size = 7;

  private final String VERSION = "1.7";
  private final TcpNode tcpNode = new TcpNode();
  private String roomID = "";
  private final String nickname = "four";
  private String sessionID = "&4()xr";
  private char[][] gameState = new char[size][size];
  private Player player;
  private Token token;
  private boolean isMyMove = false;
  private int nbInfos = 0;
  private boolean isTouchEnabled = false;
  private boolean isFirst;
  private boolean isOver;
  private boolean isWinner;
  private String moveInfo = "Drag chip and release ";

  public TcpFourInARow()
  {
    super(size, size, cellZoom(60), Color.TRANSPARENT);
  }

  public void main()
  {
    getBg().clear(Color.WHITE);
    setSimulationPeriod(25);
    addActor(new Background(), new Location(3, 3));
    setPaintOrder(Background.class, Token.class);
    addTouchListener(this, GGTouch.release | GGTouch.drag);
    isTouchEnabled = false;
    tcpNode.addTcpNodeListener(this);
    connect();
    doRun();
  }

  public void init()
  {
    isFirst = isMyMove;
    removeActors(Token.class);
    removeActors(TextActor.class);
    getBg().clear();
    for (int i = 0; i < size; i++)
      for (int k = 0; k < size; k++)
        gameState[i][k] = ' ';
    if (player == Player.YELLOW)
      showToast("Game started");
    if (!isMyMove)
      showToast("Wait to play.");
    createNewToken();
    isOver = false;
  }

  public boolean touchEvent(GGTouch touch)
  {
    if (!isTouchEnabled)
    {
      if (touch.getEvent() == GGTouch.release)
        showToast("Wait to play.");
      return true;
    }
    Location mouseLoc = toLocationInGrid(touch.getX(), touch.getY());
    if (touch.getEvent() == GGTouch.release)
    {
      if (isOver)
      {
        tcpNode.sendMessage("" + Command.over);
        return true;
      }
      if (getOneActorAt(new Location(mouseLoc.x, 1), Token.class) == null)
      {
        if (token == null)  // Don't use dropped token
          return true;
        token.setX(mouseLoc.x);
        isTouchEnabled = false;
        token.setActEnabled(true);
        token = null;  // Signal start of fall down
        tcpNode.sendMessage("" + Command.move + mouseLoc.x);
      }
      else
        showToast("This column is full.");
      return true;
    }

    if (touch.getEvent() == GGTouch.drag && !isOver)
    {
      if (token != null && token.getX() != mouseLoc.x)
      {
        token.setX(mouseLoc.x);
        tcpNode.sendMessage("" + Command.drag + mouseLoc.x);
      }
    }
    return true;
  }

  private void connect()
  {
    while (roomID.length() < 3)
      roomID = requestEntry("Enter unique game room name (more than 2 characters):");
    sessionID = sessionID + roomID;
    
    showToast("Connecting to  relay...");
    tcpNode.connect(sessionID, nickname);
    GameGrid.delay(5000);  // Wait timeout
    if (tcpNode.getNodeState() != TcpNodeState.CONNECTED)
    {
     showToast("Connection to relay failed");
     addActor(new TextActor("Internet access failed.",
         Color.BLACK, Color.TRANSPARENT, 18), new Location(0, 0));
    }
  }

  public void messageReceived(String sender, String text)
  {
    char command = text.charAt(0);
    int column;
    switch (command)
    {
      case Command.start:
        player = Player.YELLOW;   // Waiting player is yellow
        init();
        break;
      case Command.move:
        column = text.charAt(1) - 48;
        token.setX(column);
        token.setActEnabled(true); // Start falling down
        token = null;  // Signal start of fall down
        break;
      case Command.drag:
        column = text.charAt(1) - 48;
        if (token != null)
          token.setX(column);
        break;
      case Command.ready:
        isTouchEnabled = true;  // Master can use mouse
        break;
      case Command.over:
        if (isWinner)
          isMyMove = false;
        else
          isMyMove = true;
        init();
        if (isWinner)
          tcpNode.sendMessage("" + Command.over);  // Confirm restart after init
        break;
    }
  }

  public void nodeStateChanged(TcpNodeState state)
  {
  }

  public void statusReceived(String text)
  {
    L.i("status " + text);
    if (text.contains("In session:--- (0)"))  // We are first player
    {
      showToast("Connected. Waiting for a partner...");
      addActor(new TextActor("Waiting in room '" + roomID + "'",
         Color.BLACK, Color.TRANSPARENT, 18), new Location(0, 0));
    }
    else if (text.contains("In session:--- (1)")) // We are second player
    {
      showToast(moveInfo);
      isMyMove = true;  // Second player starts
      isTouchEnabled = true;
      player = Player.RED;  // Second player is red
      init();
      tcpNode.sendMessage("" + Command.start);
    }
    else if (text.contains("In session:--- ")) // We are next player
    {
      showToast("Game in progress. Terminating now...");
      TcpTools.delay(4000);
      killProcess();
    }
    else if (text.equals("Disconnected:--- four")
      || text.equals("Disconnected:--- four(1)"))
    {
      showToast("Partner disconnected");
      showToast("Waiting for a partner");
      addActor(new TextActor("Waiting in room '" + roomID + "'",
         Color.BLACK, Color.TRANSPARENT, 18), new Location(0, 0));
      isTouchEnabled = false;
      removeActors(Token.class);
      getBg().clear();
      isMyMove = false;
    }
  }

  private String requestEntry(String prompt)
  {
    return GGInputDialog.show("TcpFourInARow V" + VERSION, prompt, "");
  }

  protected void tokenArrived(Location loc, Player tokenPlayer)
  {
    gameState[loc.x][loc.y] = (tokenPlayer == Player.RED ? 'r' : 'y');
    isMyMove = !isMyMove;  // Switch move order
    Player winner = checkFour();
    if (winner != null)
      gameOver(winner);
    else
    {
      if (isBoardFull())
        gameOver(null);
      else
      {
        if (isMyMove)
        {
          if (nbInfos < 2)
          {
            showToast(moveInfo);
            nbInfos++;
          }
        }
        else
        {
          tcpNode.sendMessage("" + Command.ready);
          if (nbInfos < 2)
          {
            showToast("Wait to play");
          }
        }
        createNewToken();
      }
    }
  }

  private Player checkFour()
  {
    // Convert gameState into string pattern
    StringBuffer pattern = new StringBuffer();

    // Horizontal
    for (int i = 0; i < size; i++)
    {
      for (int k = 0; k < size; k++)
        pattern.append(gameState[i][k]);
      pattern.append(',');
    }

    // Vertical
    for (int k = 0; k < size; k++)
    {
      for (int i = 0; i < size; i++)
        pattern.append(gameState[i][k]);
      pattern.append(',');
    }

    ArrayList<Location> locs;
    // Diagonal downwards
    for (int k = 1; k < 4; k++)
    {
      for (Location loc : getDiagonalLocations(new Location(0, k), true))
        pattern.append(gameState[loc.x][loc.y]);
      pattern.append(',');
    }
    for (int i = 1; i < 4; i++)
    {
      for (Location loc : getDiagonalLocations(new Location(i, 1), true))
        pattern.append(gameState[loc.x][loc.y]);
      pattern.append(',');
    }
    // Diagonal upwards
    for (int k = 4; k < 7; k++)
    {
      for (Location loc : getDiagonalLocations(new Location(0, k), false))
        pattern.append(gameState[loc.x][loc.y]);
      pattern.append(',');
    }
    for (int i = 1; i < 4; i++)
    {
      for (Location loc : getDiagonalLocations(new Location(i, 6), false))
        pattern.append(gameState[loc.x][loc.y]);
      pattern.append(',');
    }

    if (pattern.toString().contains("rrrr"))
    {
      return Player.RED;
    }
    if (pattern.toString().contains("yyyy"))
    {
      return Player.YELLOW;
    }
    return null;
  }

  private void gameOver(Player winner)
  {
    boolean tie = false;
    if (winner == null)
    {
      tie = true;
      winner = isFirst ? player : getPartner(player);
    }
    if (winner == player)
    {
      if (tie)
        showToast("It's a draw!");
      else
      {
        showToast("You won!");
        showToast("Wait partner action to play again");
      }
      isWinner = true;
    }
    else
    {
      if (tie)
        showToast("It's a draw!");
      else
      {
        showToast("You lost!");
        showToast("Click anywhere to play again.");
      }
      isTouchEnabled = true;
      isWinner = false;
    }
    isOver = true;
  }

  private void createNewToken()
  {
    if (isMyMove)
      token = new Token(this, player);
    else
      token = new Token(this, getPartner(player));
    addActor(token, new Location(6, 0), Location.SOUTH);
    setPaintOrder(Background.class, Token.class);

  }

  private boolean isBoardFull()
  {
    return getActors().size() == 43;
  }

  private Player getPartner(Player player)
  {
    return (player == Player.RED) ? Player.YELLOW : Player.RED;
  }
  
}

